@@ -7,6 +7,7 @@ import (
 
 	"github.com/go-kit/kit/log"
 	"github.com/go-kit/kit/log/level"
+	"github.com/prometheus/client_golang/prometheus"
 
 	"github.com/openshift/telemeter/pkg/authorize"
 )
@@ -23,17 +24,32 @@ type Handler struct {
 	ForwardURL string
 	client     *http.Client
 	logger     log.Logger
+
+	// Metrics.
+	forwardRequestsTotal *prometheus.CounterVec
 }
 
 // NewHandler returns a new Handler with a http client
-func NewHandler(logger log.Logger, forwardURL string) *Handler {
-	return &Handler{
+func NewHandler(logger log.Logger, forwardURL string, reg prometheus.Registerer) *Handler {
+	h := &Handler{
 		ForwardURL: forwardURL,
 		client: &http.Client{
 			Timeout: forwardTimeout,
 		},
 		logger: log.With(logger, "component", "receive/handler"),
+		forwardRequestsTotal: prometheus.NewCounterVec(
+			prometheus.CounterOpts{
+				Name: "telemeter_forward_requests_total",
+				Help: "The number of forwarded remote-write requests.",
+			}, []string{"result"},
+		),
 	}
+
+	if reg != nil {
+		reg.MustRegister(h.forwardRequestsTotal)
+	}
+
+	return h
 }
 
 // Receive a remote-write request after it has been authenticated and forward it to Thanos
@@ -59,6 +75,7 @@ func (h *Handler) Receive(w http.ResponseWriter, r *http.Request) {
 
 	resp, err := h.client.Do(req)
 	if err != nil {
+		h.forwardRequestsTotal.WithLabelValues("error").Inc()
 		level.Error(h.logger).Log("msg", "failed to forward request", "err", err)
 		http.Error(w, err.Error(), http.StatusBadGateway)
 		return
@@ -66,8 +83,11 @@ func (h *Handler) Receive(w http.ResponseWriter, r *http.Request) {
 
 	if resp.StatusCode/100 != 2 {
 		msg := "upstream response status is not 200 OK"
+		h.forwardRequestsTotal.WithLabelValues("error").Inc()
 		level.Error(h.logger).Log("msg", msg, "statuscode", resp.Status)
-		http.Error(w, msg, http.StatusBadGateway)
+		http.Error(w, msg, resp.StatusCode)
 		return
 	}
+	h.forwardRequestsTotal.WithLabelValues("success").Inc()
+	w.WriteHeader(resp.StatusCode)
 }
